/*
Copyright (c) 2014, Sean Massa
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

Neither the name of GROUPON nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

// Generated by CoffeeScript 2.0.0-beta7
void function () {
  var Changelog, commitChanges, debug, EndIf, fs, Git, Npm, openEditor, prep, prompt, test, updateVersion, verify;
  fs = require('fs');
  debug = require('debug')('publish');
  prep = require('../prep');
  verify = require('../verify');
  Changelog = require('./changelog');
  openEditor = require('./editor');
  updateVersion = require('./version');
  commitChanges = require('./commit-changes');
  Npm = require('./npm');
  Git = require('../git');
  test = require('./test');
  prompt = require('./prompt');
  EndIf = function (log) {
    var endIf;
    return endIf = function (exitCodeOrError, message) {
      if (!(null != exitCodeOrError))
        return;
      if (exitCodeOrError instanceof Error) {
        log.error(exitCodeOrError.message);
        return process.exit(1);
      } else {
        if (exitCodeOrError === 0)
          return;
        if (null != message)
          message;
        else
          message = 'exited with ' + exitCodeOrError;
        log.error(message);
        return process.exit(exitCodeOrError);
      }
    };
  };
  module.exports = function (dir, log, config, version, testCommand) {
    var changelog, endIf, git, npm;
    debug('start');
    endIf = EndIf(log);
    git = Git(dir);
    npm = Npm(dir, log);
    changelog = Changelog(dir, git);
    return verify(dir, function (error) {
      endIf(error);
      prep(dir, log, config);
      debug('ensured license headers');
      return verify(dir, function (error) {
        endIf(error);
        return test(dir, log, npm, testCommand, function (error) {
          endIf(error);
          return changelog.build(version, function (error, tempChangelog) {
            var tempChangelogPath;
            endIf(error);
            tempChangelogPath = changelog.write(tempChangelog);
            return openEditor(tempChangelogPath, function (error) {
              var tag;
              if (null != error) {
                fs.unlinkSync(tempChangelogPath);
                endIf(error);
              }
              changelog.update(tempChangelogPath);
              updateVersion(dir, version);
              tag = 'v' + version;
              return commitChanges(git, tag, function (error) {
                endIf(error);
                return git.tag(tag, function (error) {
                  endIf(error);
                  return prompt(version, function (error) {
                    endIf(error);
                    return git.branch(function (error, branch) {
                      endIf(error);
                      return git.push(branch, function (error) {
                        endIf(error);
                        return git.pushTag(tag, function (error) {
                          endIf(error);
                          return npm.publish(function (error) {
                            endIf(error);
                            return log('success!');
                          });
                        });
                      });
                    });
                  });
                });
              });
            });
          });
        });
      });
    });
  };
}.call(this);
